package ms.itsecteam.warpdrink;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.provider.ContactsContract;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;

import ms.itsecteam.warpdrink.data.DataApi;
import ms.itsecteam.warpdrink.data.rest.RestClient;
import ms.itsecteam.warpdrink.data.objects.User;
import ms.itsecteam.warpdrink.dialogs.ChargeDialog;
import ms.itsecteam.warpdrink.dialogs.NewUserDialog;
import ms.itsecteam.warpdrink.orderList.ListViewAdapter;
import ms.itsecteam.warpdrink.orderList.Order;

public class MainActivity extends ActionBarActivity implements View.OnClickListener, View.OnKeyListener, AdapterView.OnItemClickListener, DialogInterface.OnClickListener {
    private Button btnPay, btnCharge, btnLogout;
    private ImageButton btnRefresh;
    private TextView txtSum,txtCredit;
    private ListView lstOrdered;
    private AutoCompleteTextView atxvName;

    private ListViewAdapter lstAdapter;
    private ArrayAdapter atxvAdapter;
    private ArrayList<Double> order;
    private User currentUser;
    private List<User> users;
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        getWindow().addFlags(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
        setContentView(R.layout.activity_main);
        DataApi.getInstance().setMainActivity(this);

        this.txtSum = (TextView) findViewById(R.id.txtSum);
        this.txtCredit = (TextView)findViewById(R.id.txtCredit);
        this.txtCredit.setText(getResources().getString(R.string.credit,0.0));
        this.txtSum.setText(getResources().getString(R.string.total,0.0));

        this.btnLogout = (Button) findViewById(R.id.btnLogout);
        this.btnRefresh = (ImageButton) findViewById(R.id.btnRefresh);
        this.btnPay = (Button) findViewById(R.id.btnPay);
        this.btnCharge = (Button) findViewById(R.id.btnCharge);

        this.lstAdapter = new ListViewAdapter(this, R.layout.order_list_item, new ArrayList<Order>());
        this.lstOrdered = (ListView) findViewById(R.id.lstOrdered);
        this.lstOrdered.setAdapter(this.lstAdapter);


        this.users = User.getAll();
        this.atxvAdapter = new ArrayAdapter(this, android.R.layout.select_dialog_item, this.users);

        this.atxvName = (AutoCompleteTextView) findViewById(R.id.atxvName);
        this.atxvName.setAdapter(atxvAdapter);
        this.atxvName.setThreshold(1);
        this.atxvName.setOnKeyListener(this);
        this.atxvName.setOnItemClickListener(this);

        this.btnLogout.setVisibility(View.INVISIBLE);
        this.enableControls(false);


        DataApi.getInstance().refreshUserData();

    }

    private void setCurrentUser(User user) {
        if (user != null) {
            this.enableControls(true);
            this.atxvName.dismissDropDown();
            this.btnLogout.setVisibility(View.VISIBLE);
            this.txtCredit.setText(getResources().getString(R.string.credit,user.getCredit()));
            this.currentUser = user;
        }
    }

    public void refreshUsers() {
        this.atxvAdapter.clear();
        List <User> userList = User.getAll();
        ListIterator<User> userListIterator = userList.listIterator();
        while(userListIterator.hasNext()) {
            this.atxvAdapter.add(userListIterator.next());
        }
        this.users = users;
    }

    public List<User> getUsers() {

        return users;
    }

    public User getCurrentUser() {
        return this.currentUser;
    }
    private void clearCurrentUser() {
        this.atxvName.setText("");
        this.lstAdapter.clear();
        this.enableControls(false);
        this.btnLogout.setVisibility(View.INVISIBLE);
        this.txtCredit.setText(getResources().getString(R.string.credit, 0.0));
        this.txtSum.setText(getResources().getString(R.string.total, 0.0));
        this.currentUser = null;
    }
    private void enableControls(boolean enable) {
        this.atxvName.setEnabled(!enable);
        this.btnRefresh.setEnabled(!enable);
        this.btnCharge.setEnabled(enable);
        this.btnPay.setEnabled(enable);
        this.btnLogout.setEnabled(enable);
    }
    private double getTotal() {
        double ret=0.0;
        for (int i=0;i<this.lstAdapter.getCount();i++) {
            ret += this.lstAdapter.getItem(i).getValue();
        }
        return ret;
    }
    private void refreshTotalTextView() {
        this.txtSum.setText(getResources().getString(R.string.total,this.getTotal()));
    }
    public void refreshCreditTextView() {
        this.txtCredit.setText(getResources().getString(R.string.credit,this.currentUser.getCredit()));
    }
    public void removeOrderOnClickHandler(View v) {
        Order itemToRemove = (Order) v.getTag();
        this.lstAdapter.remove(itemToRemove);
        refreshTotalTextView();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnOneEuro:
                this.lstAdapter.insert(new Order(1.0), this.lstAdapter.getCount());
                refreshTotalTextView();
                break;
            case R.id.btnFiftyCent:
                this.lstAdapter.insert(new Order(0.5), this.lstAdapter.getCount());
                refreshTotalTextView();
                break;
            case R.id.btnCharge:
                new ChargeDialog(this).show();
                break;
            case R.id.btnPay:
                double total = this.getTotal();
                if (total > 0.0){
                    if (this.currentUser.getCredit()>=total){
                        new AlertDialog.Builder(this)
                                .setTitle(getResources().getString(R.string.confirm_payment_title))
                                .setMessage(getResources().getString(R.string.confirm_payment, total))
                                .setPositiveButton(android.R.string.yes, this)
                                .setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int which) {
                                    }
                                })
                                .setIcon(android.R.drawable.ic_dialog_alert)
                                .show();
                    } else {
                        Toast.makeText(this,getResources().getString(R.string.please_charge),Toast.LENGTH_LONG).show();
                    }
                }
                break;
            case R.id.btnLogout:
                clearCurrentUser();
                break;
            case R.id.btnAddUser:
                new NewUserDialog(this).show();
                break;
            case R.id.btnRefresh:
                DataApi.getInstance().refreshUserData();
                Toast.makeText(this,"Refresh successful"  ,Toast.LENGTH_LONG).show();
                break;
        }
    }

    @Override
    public boolean onKey(View v, int keyCode, KeyEvent event) {
        if (event.getAction()==KeyEvent.ACTION_DOWN && event.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
           if(this.atxvAdapter.getCount() == 1) {
               setCurrentUser(DataApi.getInstance().remoteGetUser(((User) this.atxvAdapter.getItem(0)).getUserid()));
               this.atxvName.setText(this.atxvAdapter.getItem(0).toString());
               this.atxvName.dismissDropDown();
           }
           return true;
        }
        return false;
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        setCurrentUser(DataApi.getInstance().remoteGetUser(((User) this.atxvAdapter.getItem(position)).getUserid()));
    }

    @Override
    public void onClick(DialogInterface dialog, int which) {
        this.currentUser.setCredit(this.currentUser.getCredit() - this.getTotal());
        this.currentUser.save();
        RestClient.getInstance().saveUserCredit(this.currentUser);
        this.clearCurrentUser();
    }
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_settings:
                Toast.makeText(this, "Settings selected", Toast.LENGTH_SHORT).show();
                break;
            case R.id.action_new_user:
                new NewUserDialog(this).show();
                break;
            default:
                break;
        }

        return true;
    }

}
